#!/usr/bin/env python
import os
import sys

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path = os.path.split(playbook_file)[0]

lib_path = os.path.join(playbook_path, 'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0, lib_path)
    lib_path = os.path.join(lib_path, 'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0, lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path

from utils import playbook
from linux.apache_conf_parser import ApacheConfParser


class MyPlaybook(playbook.Playbook):
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('APACHE_HOME', '/usr/local/httpd')
        self._runcontext.set_extra_var('APACHE_CONF_FILE', '/usr/local/httpd/conf/httpd.conf')

    def set_extra_vars_post(self):
        errorMsg = ''

        apache_home = self._runcontext.get_extra_var('BIN_PATH')
        if not os.path.exists(apache_home):
            errorMsg = 'APACHE_HOME <%s> is not exists.' % apache_home
            raise Exception(errorMsg)

        apache_main_conf = self._runcontext.get_extra_var('CONF_PATH')
        if not os.path.exists(apache_main_conf):
            errorMsg = 'APACHE_CONF_FILE <%s> is not exists.' % apache_main_conf
            raise Exception(errorMsg)

        parse = ApacheConfParser(apache_main_conf)

        #get ServerRoot/DocumentRoot/ScripAlias
        #APACHE_SERVERROOT/APACHE_DOCUMENTROOT/APACHE_SCRIPTALIAS
        nodes = parse.find_directive(['ServerRoot'])
        apache_serverroot = ''
        if len(nodes) == 0:
            errorMsg = 'No ServerRoot directive founded, use the <%s>.' % apache_home
            self._runcontext.log_info(errorMsg)
            apache_serverroot = apache_home
        else:
            node = nodes[0]
            apache_serverroot = node.arguments[0].strip('\"')
        self._runcontext.set_extra_var('APACHE_SERVERROOT', apache_serverroot)

        nodes = parse.find_directive(['DocumentRoot'])
        apache_documentroot = ''
        if len(nodes) == 0:
            tmpPath = os.path.join(apache_home, 'htdocs')
            errorMsg = 'No DocumentRoot directive founded, use the <%s>.' % tmpPath
            self._runcontext.log_info(errorMsg)
            apache_documentroot = tmpPath
        else:
            node = nodes[0]
            apache_documentroot = node.arguments[0].strip('\"')
        self._runcontext.set_extra_var('APACHE_DOCUMENTROOT', apache_documentroot)

        nodes = parse.find_all_directive('ScriptAlias')
        nodes.extend(parse.find_all_directive('Alias'))
        apache_scripalias = []
        for n in nodes:
            if len(n.arguments) != 2:
                errorMsg = 'Not support Alias node: %s' % n
                self._runcontext.log_info(errorMsg)
                continue
            else:
                k = n.arguments[0].strip('\"')
                v = n.arguments[1].strip('\"')
                apache_scripalias.append([k, v])
        self._runcontext.set_extra_var('APACHE_SCRIPTALIAS', apache_scripalias)

        self._runcontext.set_parameter('_apache_conf_parse', parse)

    def get_tasks(self, dir):
        ret = []
        ret.append([dir, 'r_2_1_3.py'])
        ret.append([dir, 'r_2_1_4.py'])
        ret.append([dir, 'r_2_1_5.py'])
        ret.append([dir, 'r_2_2_1.py'])
        ret.append([dir, 'r_2_2_3.py'])
        ret.append([dir, 'r_2_2_4.py'])
        ret.append([dir, 'r_2_2_5.py'])
        ret.append([dir, 'r_2_3_2.py'])
        ret.append([dir, 'r_2_3_3.py'])
        ret.append([dir, 'r_2_3_4.py'])
        ret.append([dir, 'r_2_3_5.py'])
        ret.append([dir, 'r_2_3_8.py'])
        ret.append([dir, 'r_2_3_9.py'])
        ret.append([dir, 'r_2_4_9.py'])
        ret.append([dir, 's_2_5_1.py'])
        ret.append([dir, 'r_2_6_1.py'])
        ret.append([dir, 'r_2_6_2.py'])
        ret.append([dir, 'r_2_8_1.py'])
        ret.append([dir, 'r_2_8_2.py'])
        ret.append([dir, 'r_2_8_3.py'])
        return ret


if __name__ == '__main__':

    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
