# -*- coding: UTF-8 -*-
import re
import json
import traceback


def get_tags():
    return ['2', '2.1', '2.1.2']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):

    ret_code = 0
    ret_msg = ''
    errMsg = []
    retCheckMsg = []
    httpdcfg = runcontext.get_extra_var('APACHE_CONF_FILE')
    httpdcfg = runcontext.replace_inject_characters(httpdcfg)
    runcontext.log_debug("httpdcfg: %s" % httpdcfg)
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["checkresult3"] = "failed"
    tmp_vars["checkvalue1"] = ''
    tmp_vars["checkvalue2"] = ''
    tmp_vars["checkvalue3"] = ''
    tmp_vars["scancommand1"] = r"egrep -i '^\s*User\s\s*' %s" % httpdcfg
    tmp_vars["scancommand2"] = r"egrep -i '^\s*Group\s\s*' %s" % httpdcfg

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
    runcontext.sleep()

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
    arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)
    runcontext.sleep()


    try:
        if arc1 != 0 or arc2 != 0:
            ret_code = -2
            return (-2, "Failed to invoke module.")

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]

        resultjson2 = json.loads(astdout2)
        astdout2 = resultjson2["stdout"]

        match1_found = re.match(r"User\s+(.+)", astdout1.strip(), re.I)
        match2_found = re.match(r"Group\s+(.+)", astdout2.strip(), re.I)

        retCheckMsg.append(astdout1)
        retCheckMsg.append(astdout2)

        user = ""
        group = ""

        if match1_found != None:
            tmp_vars["checkvalue1"] = match1_found.group(0)
            user = match1_found.group(1)
            if user != 'root':
                tmp_vars["checkresult1"] = "pass"
        else:
            tmp_vars["checkvalue1"] = "No User in httpd.conf"

        if match2_found != None:
            tmp_vars["checkvalue2"] = match2_found.group(0)
            group = match2_found.group(1)
            if group != 'root':
                tmp_vars["checkresult2"] = "pass"
        else:
            tmp_vars["checkvalue2"] = "No Group in httpd.conf"


        tmp_vars["scancommand3"] = r"pids=`netstat -nltp | grep httpd | awk '{print $7}' | awk -F'/' '{print $1}'`; for pid in $pids; do ps -ef | grep $pid | grep -v grep | awk '{print $1}';done"
        runcontext.set_mod_params_instance(ansible_params, "command",
                                           tmp_vars["scancommand3"])
        arc3, astdout3, astderr3 = runcontext.invoke_module(module,
                                                            ansible_params)
        runcontext.log_debug_invoke_result(arc3, astdout3, astderr3)
        runcontext.sleep()

        resultjson3 = json.loads(astdout3)
        astdout3 = resultjson3["stdout"]

        if '' == astdout3:
            tmp_vars["checkvalue3"] = "Apache process not found"
        else:
            tmp_vars["checkvalue3"] = astdout3

        list_out3 = astdout3.split("\n")
        out3_tag = True
        for out3 in list_out3:
            if out3 != user and out3 != "root":
                out3_tag = False
        if out3_tag == True:
            tmp_vars["checkresult3"] = "pass"
        else:
            ret_code = -1

        if "pass" == tmp_vars["checkresult1"] and "pass" == tmp_vars["checkresult2"] and "pass" == tmp_vars["checkresult3"]:
            tmp_vars["policyresult"] = "pass"

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
