﻿# -*- coding: UTF-8 -*-
import re
import traceback

def get_tags():
    return ['2','2.2','2.2.2']
    
def run_task(runcontext):
    rc,out=run_audit(runcontext)
    return rc,out
    
def run_audit(runcontext):

    retCode = -1
    curtemplate=runcontext.get_template()
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["checkresult1"] = "pass"
    tmpVars["checkresult2"] = "pass"
    tmpVars["checkvalue1"] = ''
    tmpVars["checkvalue2"] = ''
        
    cfg_model = runcontext.get_parameter('_apache_conf_parse')
        
    if cfg_model is None:
        retCode = -1
        retMsg = 'Parse Apache configure file failed.'
        return retCode,retMsg
    
    try:
        #get all http nodes
        nodes = []
        nodesDirectory = cfg_model.find_directive(['Directory'])
        nodesLocation = cfg_model.find_directive(['Location'])
        traceEnable = cfg_model.find_directive(['TraceEnable'])
        nodes.extend(nodesDirectory)
        nodes.extend(nodesLocation)
        runcontext.log_debug("locationlocks: %s" % str(nodes))

        result1 = []
        result2 = []
        LimitMsg = []
        TraceMsg = []
        except_flag = True
        trace_flag = True

        if 0 != len(nodes):
            except_flag = False
            trace_flag = False
            for nodeDirectory in nodes:
                for node in nodeDirectory.get_body().get_child_nodes():
                    if "LimitExcept" == node.get_name():
                        LimitMsg.append("<%s %s>" % (node.get_name(), ' '.join(node.arguments)))

                        except_flag = True
                        flag = False
                        checkDenyAll = False
                        checkRequire = False
                        args = node.arguments
                        if "DELETE" in args or "PUT" in args or "TRACE" in args or "HEAD" in args:
                            break
                        else:
                            flag = True

                        if flag:
                            if node.get_body() is None:
                                LimitMsg = "Not Exist Deny or Require."
                                break
                            for child_node in node.get_body().get_child_nodes():
                                LimitMsg.append("    %s %s" % (child_node.get_name(), ' '.join(child_node.arguments)))

                                if node.get_name() == 'Deny':
                                    if 'all' in ' '.join(node.arguments).strip().lower():
                                        checkDenyAll = True
                                if child_node.get_name() == 'Require':
                                    if 'all denied' in ' '.join(child_node.arguments).strip().lower():
                                        checkRequire = True

                        tail = "</LimitExcept>"
                        LimitMsg.append(tail)

                        if False == checkDenyAll and False == checkRequire:
                            result1.append("failed")
                        else:
                            result1.append("pass")


        if 0 == len(traceEnable):
            TraceMsg.append("LoadModule is not configured.")
            tmpVars["checkresult2"] = "failed"
        else:
            node = traceEnable[-1]
            value = node.arguments[0]
            if value.lower() != 'off':
                tmpVars["checkresult2"] = "failed"
                TraceMsg.append("%s %s" % (node.get_name(), value))
            else:
                tmpVars["checkresult2"] = "pass"
                TraceMsg.append("%s %s" % (node.get_name(), value))


        tmpVars["checkvalue1"] = '\n'.join(LimitMsg)
        tmpVars["checkvalue2"] = '\n'.join(TraceMsg)

        if "failed" in result1:
            tmpVars["checkresult1"] = "failed"

        if False == except_flag:
            tmpVars["checkresult1"] = "failed"

        if "pass" == tmpVars["checkresult1"] and "pass" == tmpVars["checkresult2"]:
            retCode = 0
            tmpVars["policyresult"] = "pass"

            
        rc,out = curtemplate.render(runcontext.get_template_file('r_2_2_2.j2'),**tmpVars)
        
        if rc == False:
            retCode = -2
            retMsg = ('Failed to render template. %s' % out)
        else:
            retMsg = out
            
    except:
        retCode = -2
        retMsg = traceback.format_exc()

    return retCode, retMsg
