# -*- coding: UTF-8 -*-
import re
import traceback


def get_tags():
    return ['2', '2.6', '2.6.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


def run_audit(runcontext):

    ret_code = -1
    ret_msg = ''

    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["checkvalue1"] = ''
    tmp_vars["checkvalue2"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -1
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        tempMsg = []
        errMsg1 = []
        errMsg2 = []
        retCheckMsg1 = []
        retCheckMsg2 = []
        retCheckMsg = []
        retNoDosevasive = []
        errMsg10 = ''

        #Methond 2
        nodesTimeout = conf_parse.find_directive(['Timeout'])

        if 0 == len(nodesTimeout):
            errMsg2.append('directive is not configured in Timeout.')
        else:
            nodeTimeout = nodesTimeout[-1]
            if 0 == len(nodeTimeout.arguments):
                errMsg2.append("%s " % (nodeTimeout.get_name()))
            elif 1 == len(nodeTimeout.arguments):
                if False == is_number(nodeTimeout.arguments[0]):
                    errMsg2.append("%s %s" % (nodeTimeout.get_name(), nodeTimeout.arguments[0]))
                else:
                    if 60 < int(nodeTimeout.arguments[0]) or 0 >= int(nodeTimeout.arguments[0]):
                        errMsg2.append("%s %s" % (nodeTimeout.get_name(), nodeTimeout.arguments[0]))
                    else:
                        retCheckMsg2.append("%s %s" % (nodeTimeout.get_name(), nodeTimeout.arguments[0]))
            else:
                errMsg2.append("%s %s" % (nodeTimeout.get_name(), ' '.join(nodeTimeout.arguments)))

        nodesKeepAlive = conf_parse.find_directive(['KeepAlive'])
        if 0 == len(nodesKeepAlive):
            errMsg2.append('KeepAlive directive is not configured.')
        else:
            nodeKeepAlive = nodesKeepAlive[-1]
            value = "".join(nodeKeepAlive.arguments).lower()
            if None == re.match(r'^on$', value):
                errMsg2.append("%s %s" % (nodeKeepAlive.get_name(), ''.join(nodeKeepAlive.arguments)))
            else:
                retCheckMsg2.append("%s %s" % (nodeKeepAlive.get_name(), ''.join(nodeKeepAlive.arguments)))

        nodesKeepAliveTimeout = conf_parse.find_directive(['KeepAliveTimeout'])
        if 0 == len(nodesKeepAliveTimeout):
            errMsg2.append('KeepAliveTimeout directive is not configured.')
        else:
            nodeKeepAliveTimeout = nodesKeepAliveTimeout[-1]
            if 0 == len(nodeKeepAliveTimeout.arguments):
                errMsg2.append("%s " % (nodeKeepAliveTimeout))
            elif 1 == len(nodeKeepAliveTimeout.arguments):
                if False == is_number(nodeKeepAliveTimeout.arguments[0]):
                    errMsg2.append("KeepAliveTimeout %s" % (nodeKeepAliveTimeout.arguments[0]))
                else:
                    if 15 < int(nodeKeepAliveTimeout.arguments[0]) or 0 >= int(nodeKeepAliveTimeout.arguments[0]):
                        errMsg2.append("KeepAliveTimeout %s" % (nodeKeepAliveTimeout.arguments[0]))
                    else:
                        retCheckMsg2.append("KeepAliveTimeout %s" % (nodeKeepAliveTimeout.arguments[0]))
            else:
                errMsg2.append('KeepAliveTimeout %s' % (' '.join(nodeKeepAliveTimeout.arguments)))


        nodesLoadmodule = conf_parse.find_directive(['LoadModule'])
        if 0 == len(nodesLoadmodule):
            errMsg2.append('LoadModule directive is not configured.')
        else:
            reqtimeout_module_list = []
            for nodeLoadmodule in nodesLoadmodule:
                value = "".join(nodeLoadmodule.arguments)
                if "reqtimeout_module" in value:
                    reqtimeout_module_list.append(nodeLoadmodule)
            if  reqtimeout_module_list:
                nodeLoadmodule = reqtimeout_module_list[-1]
                value = "".join(nodeLoadmodule.arguments)
                if "modules/mod_reqtimeout.so" not in value:
                    errMsg2.append("LoadModule %s" % (" ".join(nodeLoadmodule.arguments)))
                else:
                    retCheckMsg2.append("LoadModule %s" % (" ".join(nodeLoadmodule.arguments)))
            else:
                nodeLoadmodule = nodesLoadmodule[-1]
                errMsg2.append("LoadModule reqtimeout_module modules/mod_reqtimeout.so is not configured.")

        nodesRequestReadTimeout = conf_parse.find_directive(['RequestReadTimeout'])
        if 0 == len(nodesRequestReadTimeout):
            errMsg2.append('RequestReadTimeout directive is not configured.')
        else:
            nodeRequestReadTimeout = nodesRequestReadTimeout[-1]
            values = nodeRequestReadTimeout.arguments
            countMinRate = 0
            tempMsg.append("RequestReadTimeout ")
            tmperrMsg = []
            for value in values:
                vlist = value.split(',')
                for v in vlist:
                    if "header" in ''.join(v):
                        v = v.split('=')
                        v = v[1].split('-')
                        if 2 == len(v):
                            if is_number(v[0]) and is_number(v[1]):
                                if int(v[1]) <= int(v[0]) or (20 > int(v[1]) or 40 < int(v[1])) or (20 > int(v[0]) or 40 < int(v[0])):
                                    tmperrMsg.append("header=%s-%s " % (v[0], v[1]))
                                else:
                                    tempMsg.append("header=%s-%s " % (v[0], v[1]))
                            else:
                                tmperrMsg.append('header=%s-%s ' % (v[0], v[1]))
                        else:
                            tmperrMsg.append("header=%s " % (v[0]))

                    if "body" in ''.join(v):
                        v = v.split('=')
                        if is_number(v[1]):
                            if 0 > int(v[1]) or 20 < int(v[1]):
                                tmperrMsg.append("body=%s " % (v[1]))
                            else:
                                tempMsg.append("body=%s " % (v[1]))
                        else:
                            tmperrMsg.append("body=%s " % (v[1]))

                    if "MinRate" in ''.join(v):
                        countMinRate = countMinRate + 1
                        v = v.split('=')
                        if 1 >= len(v):
                            tmperrMsg.append("RequestReadTimeout MinRate need value!")
                        else:
                            if False == is_number(v[1]):
                                tmperrMsg.append("MinRate=%s " % (v[1]))
                            else:
                                if int(500 < int(v[1])) or 0 >= int(v[1]):
                                    tmperrMsg.append("MinRate=%s " % (v[1]))
                                else:
                                    tempMsg.append("MinRate=%s " % (v[1]))
            if 2 != countMinRate:
                tmperrMsg.append("RequestReadTimeout configuration is insecurity!")

            if 0 != len(tmperrMsg):
                errMsg2.extend(tmperrMsg)
            else:
                retCheckMsg2.extend(tempMsg)


        if 0 == len(errMsg2):
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult2"] = "pass"
            tmp_vars["checkvalue2"] = "\n".join(retCheckMsg2)
        else:
            tmp_vars["checkvalue2"] = "\n".join(errMsg2)


        curtemplate = runcontext.get_template()
        rc, out = curtemplate.render(runcontext.get_template_file('r_2_6_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
