# -*- coding: UTF-8 -*-
import os
import traceback


def get_tags():
    return ['2', '2.6', '2.6.2']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def is_number(s):

    try:
        float(s)
        return True
    except ValueError:
        return False


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        retCheckMsg = []

        nodesLimitRequestBody = conf_parse.find_directive(['LimitRequestBody'])
        if 0 == len(nodesLimitRequestBody):
            errMsg.append('LimitRequestBody directive is not configured.')
        else:
            nodeLimitRequestBody = nodesLimitRequestBody[-1]
            if 0 == len(nodeLimitRequestBody.arguments):
                errMsg.append('LimitRequestBody  ')
            else:
                if is_number(nodeLimitRequestBody.arguments[0]):
                    if int(nodeLimitRequestBody.arguments[0]) > 102400 or int(nodeLimitRequestBody.arguments[0]) < 0:
                        errMsg.append('LimitRequestBody %s' % (nodeLimitRequestBody.arguments[0]))
                    else:
                        retCheckMsg.append('LimitRequestBody %s' % (nodeLimitRequestBody.arguments[0]))
                else:
                    errMsg.append('LimitRequestBody %s' % (' '.join(nodeLimitRequestBody.arguments)))

        nodesLimitRequestFields = conf_parse.find_directive(['LimitRequestFields'])
        if 0 == len(nodesLimitRequestFields):
            errMsg.append('LimitRequestFields directive is not configured.')
        else:
            nodeLimitRequestFields = nodesLimitRequestFields[-1]
            if 0 == len(nodeLimitRequestFields.arguments):
                errMsg.append('LimitRequestFields  ')
            else:
                if is_number(nodeLimitRequestFields.arguments[0]):
                    if int(nodeLimitRequestFields.arguments[0]) > 100 or int(nodeLimitRequestFields.arguments[0]) < 0:
                        errMsg.append('LimitRequestFields %s' % (nodeLimitRequestFields.arguments[0]))
                    else:
                        retCheckMsg.append('LimitRequestFields %s' % (nodeLimitRequestFields.arguments[0]))
                else:
                    errMsg.append('LimitRequestFields %s' % (' '.join(nodeLimitRequestFields.arguments)))

        nodesLimitRequestFieldsize = conf_parse.find_directive(['LimitRequestFieldsize'])
        if 0 == len(nodesLimitRequestFieldsize):
            errMsg.append('LimitRequestFieldsize directive is not configured.')
        else:
            nodeLimitRequestFieldsize = nodesLimitRequestFieldsize[-1]
            if 0 == len(nodeLimitRequestFieldsize.arguments):
                errMsg.append('LimitRequestFieldsize  ')
            else:
                if is_number(nodeLimitRequestFieldsize.arguments[0]):
                    if int(nodeLimitRequestFieldsize.arguments[0]) > 1024 or int(nodeLimitRequestFieldsize.arguments[0]) < 0:
                        errMsg.append('LimitRequestFieldsize %s' % (nodeLimitRequestFieldsize.arguments[0]))
                    else:
                        retCheckMsg.append('LimitRequestFieldsize %s' % (nodeLimitRequestFieldsize.arguments[0]))
                else:
                    errMsg.append('LimitRequestFieldsize %s' % (' '.join(nodeLimitRequestFieldsize.arguments)))

        nodesLimitRequestline = conf_parse.find_directive(['LimitRequestline'])
        if 0 == len(nodesLimitRequestline):
            errMsg.append('LimitRequestline directive is not configured.')
        else:
            nodeLimitRequestline = nodesLimitRequestline[-1]
            if 0 == len(nodeLimitRequestline.arguments):
                errMsg.append('LimitRequestline  ')
            else:
                if is_number(nodeLimitRequestline.arguments[0]):
                    if int(nodeLimitRequestline.arguments[0]) > 512 or int(nodeLimitRequestline.arguments[0]) < 0:
                        errMsg.append('LimitRequestline %s' % (nodeLimitRequestline.arguments[0]))
                    else:
                        retCheckMsg.append('LimitRequestline %s' % (nodeLimitRequestline.arguments[0]))
                else:
                    errMsg.append('LimitRequestline %s' % (' '.join(nodeLimitRequestline.arguments)))

        nodesCoreDumpDirectory = conf_parse.find_directive(['CoreDumpDirectory'])
        if 0 == len(nodesCoreDumpDirectory):
            errMsg.append('CoreDumpDirectory directive is not configured.')
        else:
            nodeCoreDumpDirectory = nodesCoreDumpDirectory[-1]
            if 0 == len(nodeCoreDumpDirectory.arguments):
                errMsg.append("CoreDumpDirectory  ")
            else:
                if False == os.path.exists(nodeCoreDumpDirectory.arguments[0]):
                    errMsg.append("%s is not exist configured in CoreDumpDirectory." % (nodeCoreDumpDirectory.arguments[0]))
                else:
                    retCheckMsg.append('CoreDumpDirectory %s' % (''.join(nodeCoreDumpDirectory.arguments[0])))

        if 0 == len(errMsg):
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_6_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
