# -*- coding: UTF-8 -*-
import re
import json
import traceback


def get_tags():
    return ['2', '2.9', '2.9.2']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):

    ret_code = 0
    ret_msg = ''
    errMsg = []
    retCheckMsg = []
    httpdcfg = runcontext.get_extra_var('APACHE_CONF_FILE')
    httpdcfg = runcontext.replace_inject_characters(httpdcfg)
    runcontext.log_debug("httpdcfg: %s" % httpdcfg)
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''
    tmp_vars["scancommand1"] = r"egrep '^\s*Listen\s+[0-9]+\s*$' %s" % httpdcfg
    tmp_vars["scancommand2"] = r"egrep '^\s*Listen\s+[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+:[0-9]{1,5}\s*$' %s" % httpdcfg
    tmp_vars["scancommand3"] = r"egrep '^\s*Listen\s+\[.+\]:[0-9]{1,5}\s*$' %s" % httpdcfg

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
    runcontext.sleep()

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
    arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)
    runcontext.sleep()

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand3"])
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc3, astdout3, astderr3)
    runcontext.sleep()

    try:
        if arc1 != 0 or arc2 != 0 or arc3 != 0:
            ret_code = -2
            return (-2, "Failed to invoke module.")

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]

        resultjson2 = json.loads(astdout2)
        astdout2 = resultjson2["stdout"]

        resultjson3 = json.loads(astdout3)
        astdout3 = resultjson3["stdout"]
        match1_found = re.match(r"Listen\s+80", astdout1.strip())
        match2_found = re.match(r"Listen\s+\d+\.\d+\.\d+\.\d+:80", astdout2.strip())
        match3_found = re.match(r"Listen\s+\[::ffff:\d+\.\d+\.\d+\.\d+\]:80", astdout3.strip())

        retCheckMsg.append(astdout1)
        retCheckMsg.append(astdout2)
        retCheckMsg.append(astdout3)
        if match1_found != None:
            errMsg.append(astdout1)

        if match2_found != None:
            errMsg.append(astdout2)

        if match3_found != None:
            errMsg.append(astdout3)

        ret_code = -1
        tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
        else:
            ret_code = -1

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_9_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
