﻿# -*- coding: UTF-8 -*-
import traceback

def get_tags():
    return ['2', '2.1', '2.1.3']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        corrMsg = []

        #get all ServerSignature  nodes
        nodeServerSignature = conf_parse.find_directive(['ServerSignature'])
        if len(nodeServerSignature) == 0:
            corrMsg.append('')
        else:
            node = nodeServerSignature[-1]
            value = node.arguments[0]
            if value.lower() != 'off':
                errMsg.append("%s %s" % (node.get_name(), value))
            else:
                corrMsg.append("%s %s" % (node.get_name(), value))

        #get all ServerTokens  nodes
        nodeServerTokens = conf_parse.find_directive(['ServerTokens'])
        if len(nodeServerTokens) == 0:
            errMsg.append('')
        else:
            node = nodeServerTokens[-1]
            value = node.arguments[0]
            if 'prod' == value.lower() or 'prod(uctonly)' == value.lower() or 'prod[uctonly]' == value.lower():
                corrMsg.append("%s %s" % (node.get_name(), value))
            else:
                errMsg.append("%s %s" % (node.get_name(), value))

        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(corrMsg)
        else:
            ret_code = -1
            if len(''.join(errMsg)) == 0:
                tmp_vars["checkvalue1"] = '\n'.join(corrMsg)
            else:
                tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_3.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
