# -*- coding: UTF-8 -*-
import re
import json
import traceback


def get_tags():
    return ['2', '2.1', '2.1.5']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    rc, out = run_audit(runcontext, ansible_params)
    return rc, out


def run_audit(runcontext, ansible_params):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)
    module = runcontext.get_module_file("shell.py")
    tmp_vars["scancommand1"] = ''' rpm -q httpd-manual 2>&1 '''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []

        # get all LocationMatch nodes
        nodeLocationMatchs = conf_parse.find_directive(['LocationMatch'])
        for node in nodeLocationMatchs:
            for arg in node.arguments:
                if "^/+$" in arg:
                    errMsg.append("<LocationMatch \"^/+$\">\n    ...\n</LocationMatch>")
                    break

        runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
        arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
        runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
        runcontext.sleep()
        if arc1 != 0:
            return (-2, "Failed to invoke module: %s." % astdout1)

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]
        if re.match(".*not installed", astdout1) == None:
            errMsg.append("%s." % (''.join(astdout1)))

        nodeLocations = conf_parse.find_directive(['Location'])
        for nodel in nodeLocations:
            for arg2 in nodel.arguments:

                if "server-status" in arg2:
                    errMsg.append("<Location %s>\n    ...\n<Location>" % ("".join(arg2)))

                if 'server-info' in arg2:
                    errMsg.append("<Location %s>\n    ...\n<Location>" % ("".join(arg2)))

                if 'perl-status' in arg2:
                    errMsg.append("<Location %s>\n    ...\n<Location>" % ("".join(arg2)))

        if len(errMsg) == 0:
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = ''
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_5.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
