﻿# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.2', '2.2.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:

        errMsg = []
        resultList = []
        retCheckMsg = []
        #get all Directory  nodes
        nodeDirectorys = conf_parse.find_directive(['Directory'])
        #get directory / check if have Deny from all/Require all denied
        isHaveRequire = False
        isHaveDenyfromall = False
        isHaveDenyOther = False
        isHaveallow = False
        allowMsg = []
        denyOtherMsg = []
        for node in nodeDirectorys:
            sectionName = ' '.join(node.arguments)
            sectionName.strip('\"')
            tmpMsg = []
            if '/' == sectionName:
                tmpMsg.append("<Directory />")
                for node1 in node.get_body().get_child_nodes():
                    # Apache 2.4 check if Require all denied
                    if 'require' == node1.get_name().lower():
                        if len(node1.arguments) >= 2:
                            if 'all' == node1.arguments[0].lower() and 'denied' == node1.arguments[1].lower():
                                isHaveRequire = True
                                tmpMsg.append('    %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                    elif 'allow' == node1.get_name().lower():
                        isHaveallow = True
                        errMsg.append('   %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                        allowMsg.append('    %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                    # Apache 2.2 check if deny from all
                    elif 'deny' == node1.get_name().lower():
                        if len(node1.arguments) >= 2:
                            if 'from' == node1.arguments[0].lower() and 'all' == node1.arguments[1].lower():
                                isHaveDenyfromall = True
                                tmpMsg.append('    %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                            else:
                                isHaveDenyOther = True
                                denyOtherMsg.append('    %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                        else:
                            isHaveDenyOther = True
                            denyOtherMsg.append('    %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                            errMsg.append('  %s %s' % (node1.get_name(), ' '.join(node1.arguments)))
                tmpMsg.extend(allowMsg)
                tmpMsg.extend(denyOtherMsg)

                # logic
                if isHaveDenyOther or isHaveallow:
                    resultList.append('False')
                else:
                    if isHaveRequire and isHaveDenyfromall:
                        resultList.append('False')
                        tmpMsg.append('    Deny from all\nRequire all denied')
                        errMsg.append('    Deny from all\nRequire all denied')
                    elif isHaveRequire or isHaveDenyfromall:
                        resultList.append('True')
                    else:
                        resultList.append('False')
                        errMsg.append('<Directory />\n    ...\n</Directory>')

                tmpMsg.append('</Directory>')
                # init after  Directory / check
                isHaveRequire = False
                isHaveDenyfromall = False
                isHaveDenyOther = False
                isHaveallow = False
                retCheckMsg.extend(tmpMsg)

        if 0 != len(resultList):
            if 0 == len(errMsg) and 'False' not in resultList:
                ret_code = 0
                tmp_vars["policyresult"] = "pass"
                tmp_vars["checkresult1"] = "pass"
                tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
            else:
                ret_code = -1
                tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '<Directory !/>\n  ... \n</Directory>'

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_2_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
