﻿# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.2', '2.2.3']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        retCheckMsg = []
        tmpMsg = ''
        flag1 = False
        flag2 = False
        flag3 = False
        flag4 = False

        #get all LoadModule  nodes
        nodeLoadModules = conf_parse.find_directive(['LoadModule'])
        RewriteEngines = conf_parse.find_directive(['RewriteEngine'])
        RewriteConds = conf_parse.find_directive(['RewriteCond'])
        RewriteRules = conf_parse.find_directive(['RewriteRule'])
        nodeVirtualHosts = conf_parse.find_directive(['VirtualHost'])

        if len(nodeLoadModules) == 0:
            errMsg.append('LoadModule directive is not configured')
        else:
            for loadmule in nodeLoadModules:
                args = loadmule.arguments

                if "rewrite_module" in args and 'modules/mod_rewrite.so' in args:
                    flag1 = True
                    tmpMsg = ' '.join(args)
            if not flag1:
                errMsg.append('LoadModule rewrite_module modules/mod_rewrite.so: directive is not configured')
            else:
                retCheckMsg.append('LoadModule %s' % tmpMsg)

        if len(RewriteEngines) == 0:
            errMsg.append('RewriteEngine directive is not configured')
        else:
            tmpMessageList = []
            for rewrite in RewriteEngines:
                for rw in rewrite.arguments:
                    if "on" == rw.lower():
                        flag2 = True
                        tmpMsg = ''.join(rw)
                    else:
                        tmpMessageList.append("%s %s" % (rewrite.get_name(), ''.join([str(x).strip("\"") for x in rewrite.arguments])))
            if not flag2:
                for tmpMessage in tmpMessageList:
                    errMsg.append(tmpMessage)
            else:
                retCheckMsg.append('RewriteEngine %s' % tmpMsg)

        if len(RewriteConds) == 0:
            errMsg.append('RewriteCond directive is not configured')
        else:
            tmpMessageList = []
            for rewritecond in RewriteConds:
                if "%{THE_REQUEST}" in rewritecond.arguments and "!HTTP/1\.1$" in rewritecond.arguments:
                    flag3 = True
                    tmpMsg = ' '.join(rewritecond.arguments)
                else:
                    tmpMessageList.append("%s %s" % (rewritecond.get_name(), ''.join([str(x).strip("\"") for x in rewritecond.arguments])))
            if not flag3:
                for tmpMessage in tmpMessageList:
                    errMsg.append(tmpMessage)
            else:
                retCheckMsg.append('RewriteCond %s' % tmpMsg)

        if len(RewriteRules) == 0:
            errMsg.append('RewriteRule directive is not configured')
        else:
            tmpMessageList = []
            for RewriteRule in RewriteRules:
                if ".*" in RewriteRule.arguments and "-" in RewriteRule.arguments and "[F]" in RewriteRule.arguments:
                    flag4 = True
                    tmpMsg = ' '.join(RewriteRule.arguments)
                else:
                    tmpMessageList.append("%s %s" % (RewriteRule.get_name(), ''.join([str(x).strip("\"") for x in RewriteRule.arguments])))
            if not flag4:
                for tmpMessage in tmpMessageList:
                    errMsg.append(tmpMessage)
            else:
                retCheckMsg.append('RewriteCond %s' % tmpMsg)

        errMsg2 = []
        tmpCheckMsgEngine = []
        tmpCheckMsgOptions = []
        tmpret_msgEngine = []
        tmpret_msgOptions = []
        findRewriteEngine = False
        findRewriteOptions = False
        head = ''
        tail = ''
        for nodevh in nodeVirtualHosts:
            bnodes = nodevh.get_body().get_child_nodes()

            if len(bnodes) == 0:
                errMsg2.append('VirtualHost directive has no child directives')
                continue

            errMsg2 = []
            findRewriteEngine = False
            findRewriteOptions = False
            head = '<%s %s>' % (nodevh.get_name(), ' '.join(nodevh.arguments))
            tail = '</%s>' % (nodevh.get_name())
            for cnode in bnodes:

                if cnode.get_name() == 'RewriteEngine':
                    tmpCheckMsgEngine = []
                    tmpret_msgEngine = []
                    findRewriteEngine = True
                    if "on" != ''.join(cnode.arguments).lower():
                        findRewriteEngine = False
                        tmpCheckMsgEngine.append("    %s  %s" % (cnode.get_name(), ' '.join(cnode.arguments)))
                    else:
                        tmpret_msgEngine.append("<%s %s>\n    %s  %s" % (nodevh.get_name(), ' '.join(nodevh.arguments), cnode.get_name(), ' '.join(cnode.arguments)))

                if cnode.get_name() == 'RewriteOptions':
                    tmpCheckMsgOptions = []
                    tmpret_msgOptions = []
                    findRewriteOptions = True
                    if "inherit" != ''.join(cnode.arguments).lower():
                        findRewriteOptions = False
                        tmpCheckMsgOptions.append("    %s  %s" % (cnode.get_name(), ' '.join(cnode.arguments)))
                    else:
                        tmpret_msgOptions.append("    %s  %s\n</%s>" % (cnode.get_name(), ' '.join(cnode.arguments), nodevh.get_name()))

            if False == findRewriteEngine or False == findRewriteOptions:
                if 0 == len(tmpCheckMsgEngine) and 0 == len(tmpCheckMsgOptions):
                    errMsg2.append(head)
                    errMsg2.append(tail)
                else:
                    errMsg2.append(head)
                    errMsg2.extend(tmpCheckMsgEngine)
                    errMsg2.extend(tmpCheckMsgOptions)
                    errMsg2.append(tail)
            else:
                retCheckMsg.extend(tmpret_msgEngine)
                retCheckMsg.extend(tmpret_msgOptions)

            if 0 != len(errMsg2):
                errMsg.extend(errMsg2)
                retCheckMsg.extend(errMsg2)

        tmp_vars["checkvalue1"] = '\n'.join(errMsg)
        ret_code = -1
        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_2_3.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
