﻿# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.2', '2.2.4']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = -1
    ret_msg = ''
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg
    try:
        errMsg = []
        retCheckMsg = []
        flag_400 = False
        flag_401 = False
        flag_403 = False
        flag_404 = False
        flag_405 = False
        flag_500 = False

        node_error_documents = conf_parse.find_all_directive('ErrorDocument')
        if len(node_error_documents) == 0:
            errMsg.append('ErrorDocument directive is not configured')
        else:
            for node in node_error_documents:
                args = node.arguments
                v = args[0]
                if v == '400':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_400 = True
                elif v == '401':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_401 = True
                elif v == '403':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_403 = True
                elif v == '404':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_404 = True
                elif v == '405':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_405 = True
                elif v == '500':
                    retCheckMsg.append("\n".join(node.get_lines()))
                    flag_500 = True
                else:
                    errMsg.append("\n".join(node.get_lines()))

        if flag_400 and flag_401 and flag_403 and flag_404 and flag_405 and flag_500:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
        else:
            if errMsg:
                retCheckMsg.extend(errMsg)
            tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_2_4.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out
    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
