# -*- coding: UTF-8 -*-
import os
import traceback


def get_tags():
    return ['2', '2.3', '2.3.9']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        retCheckMsg = []
        nodesDirectory = conf_parse.find_directive(['Directory'])
        if 0 == len(nodesDirectory):
            errMsg.append('The Directory directive is not configured.')
            retCheckMsg.append('The Directory directive is not configured.')
        else:
            findFlag = False
            errMsg1 = ''
            for nodeDirectory in nodesDirectory:
                if '/' in [os.path.normpath(x.strip('\"')) for x in nodeDirectory.arguments]:
                    findFlag = False
                    for node in nodeDirectory.get_body().get_child_nodes():
                        if 'AllowOverride' == node.get_name():
                            errMsg1 = ''
                            findFlag = True
                            if 0 == len(node.arguments):
                                findFlag = False
                                errMsg1 = '%s ' % (node.get_name())
                            elif 1 == len(node.arguments):
                                if 'none' not in [x.lower() for x in node.arguments]:
                                    findFlag = False
                                    errMsg1 = '%s %s' % (node.get_name(), ' '.join(node.arguments))
                            else:
                                findFlag = False
                                errMsg1 = '%s %s' % (node.get_name(), ' '.join(node.arguments))
            if 0 != len(errMsg1):
                errMsg.append(errMsg1)
                retCheckMsg.append('<Directory /> \n    %s\n</Directory>' % (errMsg1))
            if False == findFlag and 0 == len(errMsg):
                errMsg.append('  The AllowOverride directive is not found in Directory.')
                retCheckMsg.append('<Directory /> \n</Directory>')

        ret_code = -1
        tmp_vars["checkvalue1"] = '\n'.join(retCheckMsg)
        if 0 == len(errMsg):
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '<Directory /> \nAllowOverride None\n</Directory>'
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        curtemplate = runcontext.get_template()
        rc, out = curtemplate.render(runcontext.get_template_file('r_2_3_9.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
