# -*- coding: UTF-8 -*-
import os
import traceback


def get_tags():
    return ['2', '2.8', '2.8.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []

        #get all Directory nodes
        nodesLoadModule = conf_parse.find_directive(['LoadModule'])
        if 0 == len(nodesLoadModule):
            errMsg.append("LoadModule is not configured.")
        else:
            for nodeLoadModule in nodesLoadModule:
                modList = nodeLoadModule.arguments
                if 2 == len(modList):
                    modL = modList[1]
                    if 'mod_auth' in modL:
                        modL = os.path.basename(modL)
                        modL = os.path.splitext(modL)[0]
                        if modL not in ['mod_authn_core', 'mod_authn_file', 'mod_authz_core', 'mod_authz_host', 'mod_authz_groupfile', 'mod_authz_user', ]:
                            errMsg.append("LoadModule %s" % (' '.join(modList)))
                            runcontext.log_debug('LoadModule configuration is unsafe:' + "LoadModule %s" % (' '.join(modList)))
                else:
                    errMsg.append('LoadModule %s %s' % (nodeLoadModule, ' '.join(modList)))
                    runcontext.log_debug('LoadModule %s %s' % (nodeLoadModule, ' '.join(modList)))

        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = ''
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_8_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
