﻿# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.8', '2.8.2']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        corrMsg = []
        isConfigured = False

        #get all LoadModule  nodes
        nodeLoadModules = conf_parse.find_directive(['LoadModule'])
        if len(nodeLoadModules) == 0:
            errMsg.append('The LoadModule directive is not configured.')
        else:

            for modules in nodeLoadModules:
                name = modules.get_name()
                modules = modules.arguments
                strMsg = "%s %s" % (name, ' '.join(modules))
                if 'log_config_module' in modules:
                    isConfigured = True
                    if 'modules/mod_log_config.so' not in modules:
                        errMsg.append(strMsg)
                    else:
                        corrMsg.append(strMsg)

            if not isConfigured:
                errMsg.append('')

        runcontext.log_debug("errMsg: %s  corrMsg: %s" % (errMsg, corrMsg))
        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(corrMsg)
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_8_2.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
