﻿# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.8', '2.8.3']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):

    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []

        #get all LoadModule  nodes
        nodeLoadModules = conf_parse.find_directive(['LoadModule'])
        if len(nodeLoadModules) == 0:
            errMsg.append('The LoadModule directive is not configured.')
        else:

            for modules in nodeLoadModules:
                name = modules.get_name()
                modules = modules.arguments
                if 'dav_module' in modules and 'modules/mod_dav.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'dav_fs_module' in modules and 'modules/mod_dav_fs.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'status_module' in modules and 'modules/mod_status.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'autoindex_module' in modules and 'modules/mod_autoindex.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_module' in modules and 'modules/mod_proxy.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_connect_module' in modules and 'modules/mod_proxy_connect.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_ftp_module' in modules and 'modules/mod_proxy_ftp.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_http_module' in modules and 'modules/mod_proxy_http.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_fcgi_module' in modules and 'modules/mod_proxy_fcgi.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_scgi_module' in modules and 'modules/mod_proxy_scgi.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_ajp_module' in modules and 'modules/mod_proxy_ajp.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_balancer_module' in modules and 'modules/mod_proxy_balancer.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'proxy_express_module' in modules and 'modules/mod_proxy_express.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'userdir_module' in modules and 'modules/mod_userdir.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))
                if 'info_module' in modules and 'modules/mod_info.so' in modules:
                    errMsg.append("%s %s" % (name, ' '.join(modules)))

        if len(errMsg) == 0:
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = ''
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('r_2_8_3.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
