# -*- coding: UTF-8 -*-
import traceback


def get_tags():
    return ['2', '2.5', '2.5.1']


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    ret_code = 0
    ret_msg = ''

    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkvalue1"] = ''

    conf_parse = runcontext.get_parameter('_apache_conf_parse')

    if conf_parse is None:
        ret_code = -2
        ret_msg = 'Parse apache configure file failed.'
        return ret_code, ret_msg

    try:
        errMsg = []
        ret_check_msg = []

        # check log if open
        # get all LogLevel  nodes
        nodesLogLevel = conf_parse.find_directive(['LogLevel'])
        if 0 == len(nodesLogLevel):
            errMsg.append('LogLevel directive is not configured.')
        else:
            nodeLogLevel = nodesLogLevel[-1]
            if 0 == len(nodeLogLevel.arguments):
                errMsg.append('%s  ' % nodeLogLevel.get_name())
            else:
                if 'notice' not in nodeLogLevel.arguments:
                    errMsg.append('%s %s' % (nodeLogLevel.get_name(), ' '.join(nodeLogLevel.arguments)))
                else:
                    ret_check_msg.append("\n".join(nodeLogLevel.get_lines()))

        # get all ErrorLog  nodes
        nodesErrorLog = conf_parse.find_directive(['ErrorLog'])
        if 0 == len(nodesErrorLog):
            errMsg.append('ErrorLog directive is not configured.')
        else:
            nodeErrorLog = nodesErrorLog[-1]
            if 0 == len(nodeErrorLog.arguments):
                errMsg.append('%s  ' % nodeErrorLog.get_name())
            else:
                for value in [x.strip('\"') for x in nodeErrorLog.arguments]:
                    if value is None:
                        errMsg.append('%s %s' % (nodeErrorLog.get_name(), ' '.join(nodeErrorLog.arguments)))
                ret_check_msg.append("\n".join(nodeErrorLog.get_lines()))

        # get all Customlog  nodes
        nodesCustomlog = conf_parse.find_all_directive('CustomLog')
        nickName = []
        if 0 == len(nodesCustomlog):
            errMsg.append('Customlog directive is not configured.')
        else:
            nodeCustomlog = ""
            for nodeCustomlog in nodesCustomlog:
                if 0 == len(nodeCustomlog.arguments):
                    errMsg.append('%s  ' % nodeCustomlog.get_name())
                else:
                    if 2 > len(nodeCustomlog.arguments):
                        errMsg.append('%s %s' % (nodeCustomlog.get_name(), ''.join(nodeCustomlog.arguments)))
                    elif 2 == len(nodeCustomlog.arguments):
                        if '\"' != nodeCustomlog.arguments[1][0]:
                            nickName.append(nodeCustomlog.arguments[1])
                        else:
                            errMsg.append('%s %s' % (nodeCustomlog.get_name(), ' '.join(nodeCustomlog.arguments)))
                    else:
                        nodeCustomlogParam1 = nodeCustomlog.arguments[0]
                        nodeCustomlog.arguments.pop(0)
                        tmpIndex = 0
                        for node in nodeCustomlog.arguments:
                            if 'env=' in node or 'expr=' in node:
                                tmpIndex = nodeCustomlog.arguments.index(node)
                        if 0 != tmpIndex:
                            for i in range(tmpIndex, len(nodeCustomlog.arguments) - 1):
                                nodeCustomlog.arguments.pop(i)
                        xList = []
                        xListCopy = []
                        for x in nodeCustomlog.arguments:
                            xListCopy.append(x)
                            xList.append(x.strip('\"'))
                        if '%h' in xList and '%l' in xList and '%t' in xList and '\\\"%r\\' in xList and '%>s' in xList and '%b' in xList and '\\\"%{Accept}i\\' in xList and '\\\"%{Referer}i\\' in xList and '\\\"%{User-Agent}i\\' in xList:
                            pass
                        else:
                            if '\"' != xListCopy[len(xListCopy) - 1][-2:]:
                                xListCopy[len(xListCopy) - 1] = xListCopy[len(xListCopy) - 1].strip('\"')
                            if 0 == xListCopy[0].find('\"'):
                                xListCopy[0] = xListCopy[0][1:len(xListCopy[0])]
                            errMsg.append('%s %s %s ' % (nodeCustomlog.get_name(), nodeCustomlogParam1, ' '.join(xListCopy)))
            if "combined" in nickName:
                ret_check_msg.append("\n".join(nodeCustomlog.get_lines()))
            else:
                errMsg.append('Customlog directive is not configured.')

        # get all LogFormat  nodes
        nodesLogFormat = conf_parse.find_all_directive('LogFormat')
        if 0 == len(nodesLogFormat):
            errMsg.append('LogFormat directive is not configured.')
        else:
            nickNameLogFormat = []  # nickName in LogFormat;if LogFormat meet the requirements True append;else False append. nickName not in,do nothing
            for nodeLogFormat in nodesLogFormat:
                if 0 == len(nodeLogFormat.arguments):
                    errMsg.append('%s  ' % nodeLogFormat.get_name())
                else:
                    if 0 == len(nodeLogFormat.arguments):
                        errMsg.append('%s  ' % nodeLogFormat.get_name())
                    else:
                        if "combined" in nodeLogFormat.arguments:
                            xList = []
                            for x in nodeLogFormat.arguments:
                                xList.append(x.strip('\"'))
                            if '%h' in xList and '%l' in xList and '%t' in xList and '\\\"%r\\' in xList and '%>s' in xList and '%b' in xList and '\\\"%{Accept}i\\' in xList and '\\\"%{Referer}i\\' in xList and '\\\"%{User-Agent}i\\' in xList:
                                ret_check_msg.append("\n".join(nodeLogFormat.get_lines()))
                                nickNameLogFormat.append(True)
                            else:
                                nickNameLogFormat.append(False)
            if 0 == len(nickNameLogFormat):
                errMsg.append('combined is not found in any LogFormat')
            else:
                if False in nickNameLogFormat:
                    errMsg.append("LogFormat incorrect configuration")

        if 0 == len(errMsg):
            ret_code = 0
            tmp_vars["policyresult"] = "pass"
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["checkvalue1"] = '\n'.join(ret_check_msg)
        else:
            ret_code = -1
            tmp_vars["checkvalue1"] = '\n'.join(errMsg)

        rc, out = curtemplate.render(runcontext.get_template_file('s_2_5_1.j2'), **tmp_vars)

        if rc == False:
            ret_code = -2
            ret_msg = ('Failed to render template. %s' % out)
        else:
            ret_msg = out

    except:
        ret_code = -2
        ret_msg = traceback.format_exc()

    return ret_code, ret_msg
